
__version__ = "2020.6.4"
__author__ = "Xcodz"


Text_To_RgbGrayScale = {
    "0": (0, 0, 0),
    "1": (28, 28, 28),
    "2": (56, 56, 56),
    "3": (85, 85, 85),
    "4": (113, 113, 113),
    "5": (141, 141, 141),
    "6": (170, 170, 170),
    "7": (198, 198, 198),
    "8": (226, 226, 226),
    "9": (255, 255, 255)
}


class BitMap():
    def __init__(self, x, y):
        self.buffer = [[(255, 255, 255) for _ in range(x)] for _ in range(y)]

    def __repr__(self):
        return '\n'.join([''.join(x) for x in self.buffer])

    def __getitem__(self, i):
        return self.buffer[i[1]][i[0]]

    def __setitem__(self, i, sr):
        ln = sr.split('\n')
        for y in range(len(ln)):
            for x in range(len(ln[y])):
                try:
                    self.buffer[y + i[1]][x + i[0]] = ln[y][x]
                except:
                    pass


class BitMapPortion(BitMap):
    def __init__(self, bitmap, x, y, w, h):
        self.bmap = bitmap
        self.x = x
        self.y = y
        self.h = h
        self.w = w

    def __repr__(self):
        portion = self.bmap.buffer[self.y:self.h + self.y]
        for x in range(len(portion)):
            portion[x] = portion[x][self.x:self.x + self.w]
        return '\n'.join([''.join(x) for x in portion])

    def __getitem__(self, i):
        return self.bmap[
            self.x + i[0] if i[0] < self.w else self.w + self.x, self.y + i[1] if i[1] < self.h else self.h + self.y]

    def __setitem__(self, i, sr):
        self.bmap[self.x + i[0] if i[0] < self.w else self.w + self.x, self.y + i[1] if i[
                                                                                            1] < self.h else self.h + self.y] = '\n'.join(
            [''.join(x) for x in [list(xyz[0:self.w - i[0]]) for xyz in sr.split('\n')[0:self.h - i[1]]]])



from denverapi import bdtpfserv
from denverapi import bcli
import argparse
import os

__version__ = "1.0.0"
__author__ = "Xcodz"

if __name__ == '__main__':
    cli = bcli.new_cli()

    parser = argparse.ArgumentParser()

    commands = parser.add_subparsers(dest="command", required=True)
    host = commands.add_parser("host", description="Host your very own file server")
    get = commands.add_parser("get", description="Get a file from server")
    post = commands.add_parser("post", description="Post a file to server")
    new = commands.add_parser("new_directory", description="Create a directory")
    dir_lister = commands.add_parser("list", description="List a directory")

    get.add_argument("-f", "--file", help="Path to file to get", required=True)
    get.add_argument("-i", "--ip-address", help="IP Address of server", default="127.0.0.1")
    get.add_argument("-p", "--port", help="Port to the server", required=True, type=int)
    get.add_argument("-s", "--store", help="File to store, defaults to file's base name", default=None)

    post.add_argument("-f", "--file", help="File to post", required=True)
    post.add_argument("-i", "--ip-address", help="IP Address of server", default="127.0.0.1")
    post.add_argument("-p", "--port", help="Port to the server", required=True, type=int)
    post.add_argument("-s", "--store", help="Path for the file to be stored on server, defaults to file's basename",
                      default=None)

    new.add_argument("-d", "--directory", help="Directory Name", required=True)
    new.add_argument("-i", "--ip-address", help="IP Address of server", default="127.0.0.1")
    new.add_argument("-p", "--port", help="Port to the server", required=True, type=int)

    dir_lister.add_argument("-i", "--ip-address", help="IP Address of server", default="127.0.0.1")
    dir_lister.add_argument("-p", "--port", help="Port to the server", required=True, type=int)
    dir_lister.add_argument("-d", "--directory", help="Directory to list", required=True)

    host.add_argument("-i", "--ip-address", help="IP Address of server", default="127.0.0.1")
    host.add_argument("-p", "--port", help="Port to the server", required=True, type=int)
    host.add_argument("-e", "--post-directory", help="Allow the post of users to be limited to this directory",
                      default=None)
    host.add_argument("-d", "--directory", help="Host Directory", default=".")

    parser.add_argument("-v", "--version", action="version")
    parser.add_argument("--no-log", action="store_true", help="Do not log to stdout")

    parser.version = "Copyright (c) 2020 Xcodz. bdtpserver {}".format(__version__)

    args = parser.parse_args()

    try:
        if args.command == 'host':
            if not args.no_log:
                cli.info("Hosting Process Started")
                cli.info("IP Address :", args.ip_address)
                cli.info("Port       :", args.port)
            bdtpfserv.host(args.directory, (args.ip_address, args.port), args.post_directory)
        elif args.command == "get":
            if not args.no_log:
                cli.info("Getting file", args.file)
            data = bdtpfserv.get(args.file, (args.ip_address, args.port))
            if not args.no_log:
                cli.good(f"Download Complete ({len(data) // 1024} Kb)")
                cli.info("Writing File")
            file_to_write = args.store
            if file_to_write is None:
                file_to_write = os.path.basename(args.file)
            with open(file_to_write, 'w+b') as file:
                file.write(data)
            if not args.no_log:
                cli.good("File Written")
        elif args.command == "post":
            if not args.no_log:
                cli.info("Reading file")
            with open(args.file, "r+b") as file:
                data = file.read()
            if not args.no_log:
                cli.good(f"File Read ({len(data) // 1024} Kb)")
                cli.info("Posting to server")
            file_to_write = args.store
            if file_to_write is None:
                file_to_write = os.path.basename(args.file)
            bdtpfserv.post(args.store, file_to_write, (args.ip_address, args.port))
            if not args.no_log:
                cli.good("Done Posting")
        elif args.command == "new_directory":
            if not args.no_log:
                cli.info("Making Directory")
            bdtpfserv.mkdir(args.directory, (args.ip_address, args.port))
            if not args.no_log:
                cli.good("Directory Made")
        elif args.command == "list":
            directory_list = bdtpfserv.listdir(args.directory, (args.ip_address, args.port))
            cli.info("Directory listing of \"", args.directory, '":', sep="")
            for x in directory_list:
                cli.info("\t", x)
    except KeyboardInterrupt:
        if not args.no_log:
            cli.bad("Process aborted by user")
        raise SystemExit(1)
    except Exception as exception:
        if not args.no_log:
            cli.bad("Exception", str(exception))
    if not args.no_log:
        cli.good("Process Complete")



"""
Big Data Transfer Protocol
==========================

What does it do
---------------

This protocol sends big data on a address (IPv4,port)
compressing data with gzip module to make the transfer
even faster on slow networks and less data consumption.
"""

__author__ = "Xcodz"
__version__ = "2020.6.4"

import socket
import gzip
import abc
import time
import threading

if __name__ != "__main__":
    from . import log
else:
    import log

default_buffer_size = 100000
_logger_conf = {
    "debug": False,
    "error": False,
    "warning": False,
}
Mlog = log.Logger("test", _logger_conf)


class _BaseSender(abc.ABC):
    @abc.abstractmethod
    def send(self):
        pass


class _BaseReceiver(abc.ABC):
    @abc.abstractmethod
    def recv(self):
        pass


class DataSenderHost(_BaseSender):
    def __init__(self):
        self.data: bytes = b''
        self.address: tuple = ('', 0)
        self.data_send: int = 0
        self.buffer_size: int = default_buffer_size
        self.task = False

    def send(self, connected_socket: socket.socket = None):
        if connected_socket is None:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.bind(self.address)
            s.listen(5)
            connection, _ = s.accept()
        else:
            connection = connected_socket
        edata = gzip.compress(self.data, 9)

        # Data Ready
        Mlog.debug("compressed data")

        for line in range(0, len(edata), self.buffer_size):
            lts = edata[line:line + self.buffer_size]
            connection.sendall(lts)
            self.data_send += self.buffer_size

        connection.send(b'')
        if connected_socket is None:
            connection.close()
            s.close()
        self.task = True


class DataSenderPort(_BaseSender):
    def __init__(self):
        self.data: bytes = b''
        self.address: tuple = ('', 0)
        self.data_send: int = 0
        self.buffer_size: int = default_buffer_size
        self.task = False

    def send(self, connected_socket: socket.socket = None):
        edata = gzip.compress(self.data, 9)
        Mlog.debug("compressed data")
        if connected_socket is None:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(self.address)
        else:
            s = connected_socket
        for x in range(0, len(edata), self.buffer_size):
            lts = edata[x:x + self.buffer_size]
            s.sendall(lts)
            self.data_send += self.buffer_size

        s.send(b'')
        if connected_socket is None:
            s.close()
        self.task = True


class DataReceiverHost(_BaseReceiver):
    def __init__(self):
        self.address: tuple = ('', 0)
        self.data_recv: int = 0
        self.buffer_size: int = default_buffer_size
        self.data = b''
        self.task = False

    def recv(self, connected_socket: socket.socket = None):
        if connected_socket is None:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.bind(self.address)
            s.listen(5)
            connection, _ = s.accept()
        else:
            connection = connected_socket
        recv_bytes = b'\0'

        while recv_bytes != b'':
            recv_bytes = connection.recv(self.buffer_size)
            self.data += recv_bytes
            self.data_recv += self.buffer_size

        Mlog.debug("Received data")
        self.data = gzip.decompress(self.data)
        Mlog.debug("Decompressed")
        if connected_socket is None:
            connection.close()
            s.close()
        self.task = True


class DataReceiverPort(_BaseReceiver):
    def __init__(self):
        self.address: tuple = ('', 0)
        self.data_recv: int = 0
        self.buffer_size: int = default_buffer_size
        self.data = b''
        self.task = False

    def recv(self, connected_socket: socket.socket = None):
        if connected_socket is None:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(self.address)
            connection = s
        else:
            connection = connected_socket
        recv_bytes = b'\0'

        while recv_bytes != b'':
            recv_bytes = connection.recv(self.buffer_size)
            self.data += recv_bytes
            self.data_recv += self.buffer_size

        Mlog.debug("Received data")

        self.data = gzip.decompress(self.data)
        Mlog.debug("Decompressed")

        if connected_socket is None:
            connection.close()
        self.task = True


def new_send_data_host(data: bytes, addr: tuple, buffer_size=None):
    sender_object = DataSenderHost()
    sender_object.data = data
    sender_object.address = addr
    if buffer_size is not None:
        sender_object.buffer_size = buffer_size
    return sender_object


def new_send_data_port(data: bytes, addr: tuple, buffer_size=None):
    sender_object = DataSenderPort()
    sender_object.data = data
    sender_object.address = addr
    if buffer_size is not None:
        sender_object.buffer_size = buffer_size
    return sender_object


def new_receive_data_host(addr: tuple, buffer_size=None):
    sender_object = DataReceiverHost()
    sender_object.address = addr
    if buffer_size is not None:
        sender_object.buffer_size = buffer_size
    return sender_object


def new_receive_data_port(addr: tuple, buffer_size=None):
    sender_object = DataReceiverPort()
    sender_object.address = addr
    if buffer_size is not None:
        sender_object.buffer_size = buffer_size
    return sender_object


def attach_speed_logger(data_object):
    spl = []

    def sps(spl, d: _BaseSender):
        old = 0
        new = 0
        spl.append(d.buffer_size)
        while not d.task:
            time.sleep(0.01)
            new = d.data_send
            spl.append(new-old)
            old = new

    def spr(spl, d: _BaseReceiver):
        old = 0
        new = 0
        spl.append(d.buffer_size)
        while not d.task:
            time.sleep(0.01)
            new = d.data_recv
            spl.append(new-old)
            old = new

    mt = threading.Thread(target=spr if isinstance(data_object, _BaseReceiver) else sps, args=(spl, data_object), daemon=True)
    mt.start()
    return spl


def launch(data_object, connected_socket=None):
    if isinstance(data_object, _BaseSender):
        data_object.send(connected_socket)
    else:
        data_object.recv(connected_socket)


def average_speed_log(spl: list):
    while spl[0] == 0:
        spl.pop(0)
    while spl[-1] == 0:
        spl.pop(-1)
    return sum(spl)/len(spl)


def main():
    print("Reading Data")
    datats = open(input("File > "), 'r+b').read()
    print("Read Data")
    print("Making Classes")
    sc = new_send_data_port(datats, ('127.0.0.1', 4623))
    rc = new_receive_data_host(('127.0.0.1', 4623))
    spl = attach_speed_logger(rc)
    from threading import Thread
    Thread(target=launch, args=(sc,)).start()
    rc.recv()
    print(len(spl))
    print(f"Data Send:\n\tlen: {len(sc.data)}\nData Received:\n\tlen: {len(rc.data)}\n\tis_equal: {rc.data == sc.data}")
    print(f"Average Speed: {average_speed_log(spl)} bytes per second")

if __name__ == "__main__":
    main()



"""
Utilities for setup

Like find package data for including all package data
into the setup.
"""

__version__ = "2020.6.8"
__author__ = "Xcodz"

import os


def find_package_data(package: str, package_name):
    files = []
    for r, d, f in os.walk(package):
        files.extend([os.path.join(r, x)[len(package) + 1:] for x in f])
    exclude_files(files, ['.py'])
    root = {}
    for x in files:
        d = x.split(os.sep)
        d.pop(-1)
        d = ".".join(d)
        root.setdefault(package_name + '.' + d, [])
        root[package_name + '.' + d].append(os.path.basename(x))
    return {k.strip('.'): v for k, v in root.items()}


def exclude_files(filespath: list, exts: list):
    rml = []
    for x in range(len(filespath)):
        if os.path.splitext(filespath[x])[1] in exts:
            rml.append(filespath[x])
    for x in rml:
        filespath.remove(x)


def concatenate_dictionaries(d1: dict, d2: dict, *d):
    base = d1
    base.update(d2)
    for x in d:
        base.update(x)
    return base


if __name__ == '__main__':
    from pprint import pprint

    os.chdir("..")
    pprint(find_package_data("denverapi", "denverapi"))



"""
Utilities for 2d graphics like degrees between 2 lines.

It uses cartesian coordinates
"""

import math


def get_angle(start_point: tuple, end_point: tuple) -> float:
    dx = end_point[0] - start_point[0]
    dy = end_point[1] - start_point[1]
    rads = math.atan2(-dy, dx)
    rads %= 2 * math.pi
    degrees = math.degrees(rads)
    return degrees


def get_end(start_point: tuple, length: int, rotation: float) -> tuple:
    x = start_point[0] + math.cos(math.radians(rotation)) * length
    y = start_point[1] + math.cos(math.radians(rotation)) * length
    return x, y



"""
================================
Beautiful Command Line Interface
================================
........
2020.6.5
........

What does it do?
================

It helps in making of beautiful
CLI with different level print
functions. Also it supports
"""

__version__ = "2020.6.5"
__author__ = "Xcodz"

import sys

try:
    from . import ctext
except ImportError:
    import ctext


class Beauty_CLI:
    """
    Beauty Command Line Interface
    =============================

    This interface is used to make the
    output text seem beautiful. there
    are 5 main beauty functions. run,
    good, bad and info are for output.
    input is for taking input from the
    stdin.
    """

    def __init__(self, file, fmt_function):
        self.file = file
        self.fmt = fmt_function

    def _print_mode(self, *obj, file=None, end='\n', sep=' ', flush=False, mode='n'):
        """
        emulates the default print function but with a different mode
        """
        if file is None:
            file = self.file
        ostr = []
        for x in obj:
            if isinstance(x, str):
                ostr.append(x)
            else:
                ostr.append(repr(x))
        if mode == 'n':
            print(*ostr, sep=sep, flush=flush, file=file, end=end)
        else:
            print(self.fmt(sep.join(ostr), mode), sep=sep, flush=flush, file=file, end=end)

    def good(self, *obj, file=None, end='\n', sep=' ', flush=False):
        """
        emulates the default print function for good mode
        """
        self._print_mode(*obj, file=file, end=end, sep=sep, flush=flush, mode='g')

    def bad(self, *obj, file=None, end='\n', sep=' ', flush=False):
        """
        emulates the default print function for bad mode
        """
        self._print_mode(*obj, file=file, end=end, sep=sep, flush=flush, mode='b')

    def info(self, *obj, file=None, end='\n', sep=' ', flush=False):
        """
        emulates the default print function for info mode
        """
        self._print_mode(*obj, file=file, end=end, sep=sep, flush=flush, mode='i')

    def input(self, prompt=None):
        """
        emulates the default input function for query mode
        """
        if prompt is None:
            prompt = ''
        self._print_mode(prompt, end='', flush=True, mode='q')
        return input()

    def run(self, *obj, file=None, end='\n', sep=' ', flush=False):
        """
        emulates the default print function for run mode
        """
        self._print_mode(*obj, file=file, end=end, sep=sep, flush=flush, mode='r')


def _fmt_bcli(text, m):
    if m == "i":
        return '\033[93m[!] ' + text + ctext.ColoredText.resetEscapeSequence['all']
    if m == "q":
        return '\033[94m[?] ' + text + ctext.ColoredText.resetEscapeSequence['all']
    if m == "b":
        return '\033[91m[-] ' + text + ctext.ColoredText.resetEscapeSequence['all']
    if m == 'g':
        return '\033[92m[+] ' + text + ctext.ColoredText.resetEscapeSequence['all']
    if m == 'r':
        return '\033[97m[~] ' + text + ctext.ColoredText.resetEscapeSequence['all']


def new_cli(file=sys.stdout, fmt='bcli') -> Beauty_CLI:
    return Beauty_CLI(file, eval(f'_fmt_{fmt}'))


def main():
    mcli = new_cli()
    mcli.info("This is 'info'")
    mcli.good("This is 'good'")
    mcli.bad("This is 'bad'")
    mcli.run("This is 'run'")
    mcli.input("This is 'input'")


if __name__ == '__main__':
    main()



from denverapi import bcli
import os

cli = bcli.new_cli()


def tool(path):
    p = os.path.basename(path)
    p = os.path.splitext(p)[0]
    return p


if __name__ == '__main__':
    directory = os.path.dirname(__file__)
    cli.info("Available Tools")
    for x in os.listdir(directory):
        if not x.startswith("_") and x.endswith(".py"):
            cli.info("\t", tool(x), sep='')



"""
Colored picture
===============

You can use this module to read, write or print images onto screen, or get printable image.
"""

try:
    from . import ctext
except ImportError:
    import ctext

fore_color = list(ctext.ColoredText.cloredTextEscapeSequenceFore.values())
back_color = list(ctext.ColoredText.cloredTextEscapeSequenceBack.values())
style = list(ctext.ColoredText.styleEscapeSequence.values())
reset = ctext.ColoredText.resetEscapeSequence["all"]


class CImage:
    def __init__(self, image_ascii: str, image_ansi: bytes):
        self.image = image_ascii
        self.image_c = image_ansi


def parse_colors(ansi: bytes) -> list:
    code = []
    for x in range(0, len(ansi), 3):
        fore_color_code = ansi[x]
        back_color_code = ansi[x + 1]
        style_code = ansi[x + 2]
        code.append(f"{fore_color[fore_color_code]}{back_color[back_color_code]}{style[style_code]}{{a}}{reset}")
    return code


def combine(image: CImage) -> str:
    text_line = ''
    ansi_code = parse_colors(image.image_c)
    for line in image.image.split("\n"):
        for character in line:
            text_line += ansi_code.pop(0).format(a=character)
        text_line += "\n"
    return text_line


def write_image(image: CImage, file: str) -> None:
    with open(file, "w+b") as writer:
        writer.write(image.image.encode("utf-8"))
        writer.write(b"\0")
        writer.write(image.image_c)


def read_image(file: str) -> CImage:
    with open(file, "r+b") as reader:
        data = reader.read()
    ascii_text, ansi_code = data.split(b"\0", 1)
    ascii_text = ascii_text.decode("utf-8")
    return CImage(ascii_text, ansi_code)


if __name__ == '__main__':

    image_ascii = """+1
    234"""

    image_ansi = b"""\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"""

    mi = CImage(image_ascii, image_ansi)
    print(combine(mi))



__version__ = "2020.6.4"


def _strip_inside_white_space(line: str):
    return ' '.join([x.strip() for x in line.split(' ')])


def _strip_comment(line: str):
    if line.find("#") == -1:
        return line
    else:
        return line[0:line.index("#")]


def analise_src_imports(src: str) -> list:
    """
    Searches for all possible imports inside the src file.

    :param src: The src code required for the analysis
    :return: the list of all the imports within the module
    """
    src_code = src.split("\n")
    imports = []
    for raw_line in src_code:
        line = _strip_inside_white_space(_strip_comment(raw_line.strip()))
        if line.startswith("from "):
            line = line.split(" ", 2)
            if line[2] == "import ":
                if line[0].startswith('.'):
                    imports.append(line[0])
        elif line.startswith("import "):
            if " as " not in line:
                line = line.split(" ", 1)
                i = [x.strip() for x in line[1].split(',')]
                imports.extend(i)
            else:
                line = line.split(" ", 2)
                imports.append(line[1])
    return list(set(imports))


def main():
    import os
    import sys
    src = [open(os.path.dirname(__file__)+'/'+x).read() if os.path.isfile(os.path.dirname(__file__)+'/'+x) else '' for x in os.listdir(os.path.dirname(__file__))]
    a_imports = [analise_src_imports(x) for x in src]
    for k, v in zip(os.listdir(os.path.dirname(__file__)), a_imports):
        print(f"In {k}: {v}")


if __name__ == '__main__':
    main()



"""
This module uses bdtp for files hosting
using a single connection method with speed
ranging up to 2.0 MB/s or more if computer have
good powered CPU
"""

__version__ = "2020.6.4"
__author__ = "Xcodz"

import socket
import threading
import pickle
import os
from . import bdtp


def get(file: str, addr: tuple):
    assert type(file) == str or type(addr) == tuple, "Invalid Parameter Types"
    request = pickle.dumps(['get', file])
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(addr)
    s.sendall(request)
    data = bdtp.new_receive_data_port(('', 0))
    data.recv(s)
    data = data.data
    return data


def post(file: str, data: bytes, addr: tuple):
    assert type(file) == str or type(addr) == tuple or type(data) == bytes, "Invalid Parameter Types"
    request = pickle.dumps(['post', file])
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(addr)
    s.sendall(request)
    s.recv(1)
    data = bdtp.new_send_data_port(data, ('', 0))
    data.send(s)


def mkdir(name: str, addr: tuple):
    assert type(name) == str or type(addr) == tuple
    request = pickle.dumps(["mkdir", name])
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(addr)
    s.sendall(request)
    s.close()


def handle_request(connection: socket.socket, addr: tuple, hdir: str, shdir: str):
    req = pickle.loads(connection.recv(1000))
    if req[0] == 'get':
        print(f"{addr} requested to get \"{req[1]}\"")
        try:
            data = open(os.path.join(hdir, req[1].replace('/', os.sep)), 'r+b').read()
            d = bdtp.new_send_data_host(data, ('', 0))
            d.send(connection)
        except Exception as e:
            print(f"{addr} get request failed: {str(e)}")
    elif req[0] == 'mkdir':
        print(f"{addr} requested to mkdir \"{req[1]}\"")
        try:
            os.mkdir(os.path.join(hdir, req[1].replace('/', os.sep)))
        except Exception as e:
            print(f"{addr} mkdir request failed: {str(e)}")
    elif req[0] == 'listdir':
        print(f"{addr} requested to listdir \"{req[1]}\"")
        try:
            r = pickle.dumps(os.listdir(os.path.join(hdir, req[1].replace('/', os.sep))))
            d = bdtp.new_send_data_host(r, ('', 0))
            d.send(connection)
        except Exception as e:
            print(f"{addr} listdir request failed: {str(e)}")
    elif req[0] == 'post':
        print(f"{addr} requested to post \"{req[1]}\"")
        try:
            connection.sendall(b'')
            d = bdtp.new_receive_data_host(('', 0))
            d.recv(connection)
            f = open(os.path.join(hdir, shdir, req[1].replace('/', os.sep)), 'w+b')
            f.write(d.data)
            f.close()
        except Exception as e:
            print(f"{addr} post request failed: {str(e)}")
    connection.close()


def listdir(dir_name: str, addr: tuple):
    if dir_name == '':
        dir_name = '.'
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(addr)
    s.sendall(pickle.dumps(['listdir', dir_name]))
    d = bdtp.new_receive_data_port(('', 0))
    d.recv(s)
    return pickle.loads(d.data)


def host(home_dir: str, addr: tuple, subdir_post: str = None):
    if subdir_post is None:
        subdir_post = home_dir
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(addr)
    s.listen(5)
    while True:
        c, addr = s.accept()
        threading.Thread(target=handle_request, args=(c, addr, home_dir, subdir_post)).start()



from __future__ import annotations

import copy
import math

__version__ = "2020.6.4"
__author__ = "Xcodz"


def flatten(x: float, y: float, z: float, scale: int, distance: int) -> tuple:
    """
    Converts 3d point to a 2d drawable point

    >>> flatten(1, 2, 3, 10, 10)
    (7.6923076923076925, 15.384615384615385)
    """
    projected_x = ((x * distance) / (z + distance)) * scale
    projected_y = ((y * distance) / (z + distance)) * scale
    return projected_x, projected_y


def model_rotate(model, axis, angle) -> list:
    """
    Rotate a model
    """
    d = copy.deepcopy(model)
    for x in range(len(d)):
        p1, p2 = d[x]
        n = (rotate(p1[0], p1[1], p1[2], axis, angle), rotate(p2[0], p2[1], p2[2], axis, angle))
        d[x] = n
    return d


def model_flatten(model, scale, distance) -> list:
    """
    flatten complete model
    """
    d = copy.deepcopy(model)
    for x in range(len(d)):
        p1, p2 = d[x]
        n = (flatten(p1[0], p1[1], p1[2], scale, distance), flatten(p2[0], p2[1], p2[2], scale, distance))
        d[x] = n
    return d


def rotate(x: int, y: int, z: int, axis: str, angle: int):
    """
    rotate a point around a certain axis with a certain angle
    angler can be any integer between 1, 360

    >>> rotate(1, 2, 3, 'y', 90)
    (3.130524675073759, 2, 0.4470070007889556)
    """
    if angle > 360 or angle < 0:
        raise ValueError("Angle is supposed to be in between 0, 360")
    if type(x) is not int:
        raise TypeError("x must be int")
    if type(y) is not int:
        raise TypeError("y must be int")
    if type(z) is not int:
        raise TypeError("z must be int")
    angle = angle / 450 * 180 / math.pi
    if axis == 'z':
        newX = x * math.cos(angle) - y * math.sin(angle)
        newY = y * math.cos(angle) + x * math.sin(angle)
        newZ = z
    elif axis == 'x':
        newY = y * math.cos(angle) - z * math.sin(angle)
        newZ = z * math.cos(angle) + y * math.sin(angle)
        newX = x
    elif axis == 'y':
        newX = x * math.cos(angle) - z * math.sin(angle)
        newZ = z * math.cos(angle) + x * math.sin(angle)
        newY = y
    else:
        raise ValueError('not a valid axis')
    nx = newX
    ny = newY
    nz = newZ
    return nx, ny, nz


class ModelMake:
    def cube(x, y, z, s=1):
        mcube = [((x, y, z), (x + s, y, z)),
                 ((x, y, z), (x, y + s, z)),
                 ((x, y, z), (x, y, z + s)),
                 ((x, y, z + s), (x + s, y, z + s)),
                 ((x, y, z + s), (x, y + s, z + s)),
                 ((x + s, y, z + s), (x + s, y, z)),
                 ((x + s, y, z + s), (x + s, y + s, z + s)),
                 ((x + s, y, z), (x + s, y + s, z)),
                 ((x, y + s, z + s), (x + s, y + s, z + s)),
                 ((x + s, y + s, z + s), (x + s, y + s, z)),
                 ((x, y + s, z + s), (x, y + s, z)),
                 ((x, y + s, z), (x + s, y + s, z))]
        return mcube


def model_dump_to_file(model_file, model):
    with open(model_file, 'w') as f:
        for segment in model:
            coord1, coord2 = segment
            f.write( "{} {} {}:{} {} {}\n".format(*coord1, *coord2))


def model_load_from_file(model_file):
    f = open(model_file).readlines()
    model = []
    for x in f:
        p1s, p2s = x.split(':', 1)
        p11, p12, p13 = p1s.split(' ', 2)
        p21, p22, p23 = p2s.split(' ', 2)
        p11, p12, p13 = float(p11), float(p12), float(p13)
        p21, p22, p23 = float(p21), float(p22), float(p23)

        n = ((p11, p12, p13), (p21, p22, p23))
        model.append(n)
    return model


def model_viewer(model):
    import pygame
    pygame.init()
    fpsclock = pygame.time.Clock()
    disp = pygame.display.set_mode((600, 400))
    cube3 = model_load_from_file(model)
    rotate = False
    rotate2 = False
    scale = 1
    distance = 10
    r = 1
    r2 = 1
    a = 1
    iskd = False
    gevent = None
    while True:
        if rotate:
            cube3 = model_rotate(cube3, "y", r * a)
            rotate = False
        if rotate2:
            cube3 = model_rotate(cube3, "x", r2 * a)
            rotate2 = False
        cube2 = model_flatten(cube3, scale, distance)
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                raise SystemExit(0)
            if event.type == pygame.KEYUP:
                iskd = False
            if event.type == pygame.KEYDOWN or iskd:
                iskd = True
                gevent = event
        if iskd and gevent != None:
            if gevent.type == pygame.KEYDOWN:
                event = gevent
                if event.key == pygame.K_UP or event.key == pygame.K_DOWN:
                    rotate2 = True
                    if event.key != pygame.K_UP:
                        r2 = 0.2
                    else:
                        r2 = -0.2
                elif event.key == pygame.K_LEFT or event.key == pygame.K_RIGHT:
                    rotate = True
                    if event.key != pygame.K_LEFT:
                        r = 0.2
                    else:
                        r = -0.2
                elif event.key == pygame.K_a:
                    scale -= 1
                elif event.key == pygame.K_d:
                    scale += 1
                elif event.key == pygame.K_w:
                    distance -= 1
                elif event.key == pygame.K_s:
                    distance += 1
                elif event.key == pygame.K_z:
                    cube3 = model_rotate(cube3, "z", 0.2 * a)
                elif event.key == pygame.K_x:
                    cube3 = model_rotate(cube3, "z", -0.2 * a)
                elif event.key == pygame.K_q:
                    a -= 1
                elif event.key == pygame.K_e:
                    a += 1
        disp.fill((255, 255, 255))

        if distance == 0:
            distance = 1
        for x, y in cube2:
            pygame.draw.aaline(disp, (0, 0, 0), (x[0] + 300, x[1] + 200), (y[0] + 300, y[1] + 200))

        pygame.display.update()
        fpsclock.tick(10)


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser("graphics3d pygame veiw")
    parser.add_argument("model", metavar="PATH_TO_MODEL", help="PATH TO MODEL")
    args = parser.parse_args()
    model_viewer(args.model)




"""
Get User input to get file, selection, etc
"""

import os

__author__ = "Xcodz"
__version__ = "2020.6.4"


def clear():
    if os.name == "nt":
        os.system('cls')
    else:
        os.system("clear")


def print_tree(path):
    dirl = []
    fill = []
    ldir = [x for x in os.listdir(path)]
    for x in ldir:
        if os.path.isdir(os.path.join(path, x)):
            dirl.append(x)
        else:
            fill.append(x)
    dirl.sort()
    fill.sort()
    tp = f'[{path}]\n'
    for x in dirl:
        tp += '\n# '+x
    for x in fill:
        tp += '\n@ '+x
    print(tp)


def get_file_path(mdir=None):
    if mdir is None:
        mdir = os.getcwd()
    mpath = os.path.abspath(mdir)
    while True:
        print_tree(mpath)
        f = input(">")
        if os.path.isfile(m := os.path.join(mpath, f)):
            clear()
            return m
        elif os.path.isdir(m := os.path.join(mpath, f)):
            mpath = os.path.abspath(m)
            clear()


def is_ipv4(ip: str):
    ipp = ip.split(".")
    if len(ipp) != 4:
        return False
    for x in ipp:
        if not x.isnumeric():
            return False
        elif not (int(x) >= 0 or int(x) < 256):
            return False
    return True


def get_ipv4_port(default = ('127.0.0.1', '8000')):
    ipv4 = input(f'IPV4 [{default[0]}] >')
    port = input(f'Port [{default[1]}] >')
    addr = []
    if is_ipv4(ipv4):
        addr.append(ipv4)
    else:
        print("Invalid IPV4, using default")
        addr.append(default[0])
    if port.isnumeric():
        addr.append(int(port))
    else:
        print("Invalid Port, using default")
        addr.append(default[1])
    return tuple(addr)


if __name__ == '__main__':
    print(get_ipv4_port())



"""
This is very useful module in-case of cipher encrypting and decrypting of data.
This contains many cipher methods. The Best of all is the vigenere cipher which
is also used in these modern times.

The Source code is taken from the book "Cracking Codes with Python". Although
hacking methods for all the ciphers are not created, the hackers can still
break the code by using different techniques.
"""

__version__ = "2020.6.4"
__author__ = "Xcodz"

import math
import cryptography.fernet
import base64


class crypto_math:
    def gcd(a, b):
        while a != 0:
            a, b = b % a, a
        return b

    def findModInverse(a, m):
        if crypto_math.gcd(a, m) != 1:
            return None
        u1, u2, u3 = 1, 0, a
        v1, v2, v3 = 0, 1, m
        while v3 != 0:
            q = u3 // v3
            v1, v2, v3, u1, u2, u3 = (u1 - q * v1), (u2 - q * v2), (u3 - q * v3), v1, v2, v3
        return u1 % m


class cVig:
    l = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'

    def translateMessage(key, message, mode):
        l = cVig.l
        t = []
        ki = 0
        k = key.upper()
        for x in message:
            n = l.find(x.upper())
            if n != -1:
                if mode == 'e':
                    n += l.find(key[ki])
                elif mode == 'd':
                    n -= l.find(key[ki])
                n %= len(l)
                if x.isupper():
                    t.append(l[n])
                elif x.islower():
                    t.append(l[n].lower())
                ki += 1
                if ki == len(key):
                    ki = 0
            else:
                t.append(x)
        return ''.join(t)


class cSub:
    l = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'

    def keyIsValid(key):
        keyList = list(key)
        lettersList = list(cSub.l)
        keyList.sort()
        return keyList == lettersList

    def translateMessage(key: str, message: str, mode):
        t = ''
        ca = cSub.l
        cb = key
        if mode == 'd':
            ca, cb = cb, ca
        for x in message:
            if x.upper() in ca:
                si = ca.find(x.upper())
                if x.isupper():
                    t += cb[si].upper()
                else:
                    t += cb[si].lower()
            else:
                t += x
        return t


class cAffine:
    def getKeyParts(key):
        a = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 !?.'
        keyA = key // len(a)
        keyB = key % len(a)
        return keyA, keyB

    def checkKeys(keyA, keyB):
        a = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 !?.'
        if keyA < 0 or keyB < 0 or keyB > len(a) - 1:
            return False
        if crypto_math.gcd(keyA, len(a)) != 1:
            return False
        return True


class crypt:
    class morse:
        table = {
            '.-': 'A',
            '-...': 'B',
            '-.-.': 'C',
            '-..': 'D',
            '.': 'E',
            '..-.': 'F',
            '--.': 'G',
            '....': 'H',
            '..': 'I',
            '.---': 'J',
            '-.-': 'K',
            '.-..': 'L',
            '--': 'M',
            '-.': 'N',
            '---': 'O',
            '.--.': 'P',
            '--.-': 'Q',
            '.-.': 'R',
            '...': 'S',
            '-': 'T',
            '..-': 'U',
            '...-': 'V',
            '.--': 'W',
            '-..-': 'X',
            '-.--': 'Y',
            '--..': 'Z',
            '.----': '1',
            '..---': '2',
            '...--': '3',
            '....-': '4',
            '.....': '5',
            '-....': '6',
            '--...': '7',
            '---..': '8',
            '----.': '9',
            '-----': '0'
        }

        def encode(st: str):
            """MORSE CODE ENCODER

A ' ' MEANS PARTITION BETWEEN LETTERS
A '/' MEANS PARTITION BETWEEN WORDS"""
            t = ''
            tb = {v: k for k, v in crypt.morse.table.items()}
            s = list(st.upper())
            for x in s:
                if x not in tb.keys() and x != ' ':
                    s.remove(x)
            w = []
            for x in s:
                if x == ' ':
                    t += ' '.join(w) + '/'
                    w = []
                else:
                    w.append(tb[x])
            t += ' '.join(w)
            return t

        def decode(s: str):
            tb = crypt.morse.table.copy()
            d = [x.split() for x in s.split('/')]
            t = ''
            for x in d:
                for y in x:
                    t += tb[y]
                t += ' '
            return t[0:-1]

    class basic:
        def encode(b: bytes = b''):
            """Encode Bytes to String"""
            d = [hex(x)[2:] for x in list(b)]
            for x in range(len(d)):
                if len(d[x]) == 1:
                    d[x] = '0' + d[x]
            return ''.join(d)

        def decode(s: str):
            return bytes([int('0x' + s[x] + s[x + 1], 0) for x in range(0, len(s), 2)])

    class cipher:
        class reverse:
            def crypt(s: str):
                """ENCODING AND DECODING FUNCTIONS ARE SAME"""
                t = ''
                i = len(s) - 1
                while i >= 0:
                    t += s[i]
                    i -= 1
                return t

        class caesar:
            def encrypt(s: str, k: int):
                """Encrypts with Caesar cipher"""
                a = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 !?.'
                t = ''
                for x in s:
                    if x in a:
                        si = a.find(x)
                        ti = si + k
                        if ti >= len(a):
                            ti -= len(a)
                        elif ti < 0:
                            ti += len(a)
                        t += a[ti]
                    else:
                        t += x
                return t

            def decrypt(s: str, k: int):
                """Decrypts with caesar cipher"""
                a = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 !?.'
                t = ''
                for x in s:
                    if x in a:
                        si = a.find(x)
                        ti = si - k
                        if ti >= len(a):
                            ti -= len(a)
                        elif ti < 0:
                            ti += len(a)
                        t += a[ti]
                    else:
                        t += x
                return t

        class transposition:
            def encrypt(message: str, key: int):
                # Each string in ciphertext represents a column in the grid:
                ciphertext = [''] * key
                # Loop through each column in ciphertext:
                for column in range(key):
                    currentIndex = column
                    # Keep looping until currentIndex goes past the message length:
                    while currentIndex < len(message):
                        # Place the character at currentIndex in message at the
                        # end of the current column in the ciphertext list:
                        ciphertext[column] += message[currentIndex]
                        # Move currentIndex over:
                        currentIndex += key
                # Convert the ciphertext list into a single string value and return it:
                return ''.join(ciphertext)

            def decrypt(message: str, key: int):
                numOfColumns = int(math.ceil(len(message) / float(key)))
                numOfRows = key
                numOfShadedBoxes = (numOfColumns * numOfRows) - len(message)
                plaintext = [''] * numOfColumns
                column = row = 0
                for symbol in message:
                    plaintext[column] += symbol
                    column += 1
                    if (column == numOfColumns) or (column == numOfColumns - 1 and row >= numOfRows - numOfShadedBoxes):
                        column = 0
                        row += 1
                return ''.join(plaintext)

        class affine:
            def encrypt(message: str, key: int):
                a = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 !?.'
                ka, kb = cAffine.getKeyParts(key)
                if cAffine.checkKeys(ka, kb):
                    ct = ''
                    for x in message:
                        if x in a:
                            si = a.find(x)
                            ct += a[(si * ka + kb) % len(a)]
                        else:
                            ct += x
                    return ct
                else:
                    return message

            def decrypt(message: str, key: int):
                a = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 !?.'
                ka, kb = cAffine.getKeyParts(key)
                if cAffine.checkKeys(ka, kb):
                    pt = ''
                    mioka = crypto_math.findModInverse(ka, len(a))
                    for x in message:
                        if x in a:
                            si = a.find(x)
                            pt += a[(si - kb) * mioka % len(a)]
                        else:
                            pt += x
                    return pt
                else:
                    return message

        class substitution:
            def encrypt(m: str, key: str):
                return cSub.translateMessage(key, m, 'e')

            def decrypt(m: str, key: str):
                return cSub.translateMessage(key, m, 'd')

        class vigenere:
            def encrypt(m: str, k: str):
                return cVig.translateMessage(k, m, 'e')

            def decrypt(m: str, k: str):
                return cVig.translateMessage(k, m, 'd')


def encrypt(data: bytes, key: bytes) -> bytes:
    machine = cryptography.fernet.Fernet(base64.urlsafe_b64encode(key))
    return machine.encrypt(data)


def decrypt(data: bytes, key: bytes) -> bytes:
    machine = cryptography.fernet.Fernet(base64.urlsafe_b64encode(key))
    try:
        return machine.decrypt(data)
    except cryptography.fernet.InvalidToken:
        return b''



"""
This module provides you with basic logging facilities which
can be used by your programs to get developed easily. while
debugging you can keep the configurations on and while
releasing you can switch of error, warnings, debug or you
can also change the file to a new file which can contain
the log.
"""
__version__ = "2020.6.4"
__author__ = "Xcodz"

import sys, os, time

conf = {
    "debug": True,
    "error": True,
    "warning": True,
}

formatables = [
    "name",
    "error",
    "msg",
    "asctime",
    "pid",
    "ppid",
    "level"]


class Logger():
    def __init__(self, name, conf=conf, file=sys.stderr):
        self.name = name
        self.conf = conf
        self.format = "{level}:{name}:{msg}\n"
        self.file = file

    def debug(self, info, error=None):
        if self.conf["debug"]:
            self.file.write(
                self.format.format(name=self.name, error=error, pid=os.getpid(), level="DEBUG", ppid=os.getppid(),
                                   msg=info, asctime=time.asctime()))
            self.file.flush()

    def error(self, info, error=None):
        if self.conf["error"]:
            self.file.write(
                self.format.format(name=self.name, error=error, pid=os.getpid(), level="ERROR", ppid=os.getppid(),
                                   msg=info, asctime=time.asctime()))
            self.file.flush()

    def warning(self, info, error=None):
        if self.conf["warning"]:
            self.file.write(
                self.format.format(name=self.name, error=error, pid=os.getpid(), level="WARNING", ppid=os.getppid(),
                                   msg=info, asctime=time.asctime()))
            self.file.flush()



"""
This Text util is useful in joining two paragraphs so the newlines
don't trouble.
"""
__version__ = "2020.6.4"
__author__ = "Xcodz"

import textwrap


class text:
    def wrap(p, w=70):
        d = [textwrap.fill(x, w) for x in p.split('\n')]
        r = []
        for x in d:
            r.extend(x.split('\n'))
        return r

    def join(p1, p2, *p, w=40, j=''):
        s1 = text.wrap(p1, w - 1)
        s2 = text.wrap(p2, w - 1)
        s = [text.wrap(x, w - 1) for x in p]
        s.insert(0, s2.copy())
        del s2
        lenl = max(*([len(s[x]) for x in range(len(s))] + [len(s1)]))
        text.vfill(s1, lenl)
        for x in range(len(s)):
            for y in range(len(s[x])):
                s[x][y] = text.linefill(s[x][y], w)
        for y in range(len(s1)):
            s1[y] = text.linefill(s1[y], w)
        for x in s: text.vfill(x, lenl)
        for x in range(len(s)):
            for y in range(len(s[x])):
                s1[y] += j + s[x][y]
        return '\n'.join(s1)

    def linefill(l, width, fill=' '):
        return l + (fill * (width - len(l)))

    def vfill(l, length, fill=''):
        l.extend([fill for _ in range(length - len(l))])



import pygame
import argparse
import denverapi

pygame.init()
pygame.key.start_text_input()

WIDTH = 720
HEIGHT = 580
FPS = 60
CELL_HEIGHT = 20
CELL_WIDTH = 12

# Colors  (R,  G,  B)
GRAY = (250, 250, 250)
BLACK = (0, 0, 0)
BLUE = (0, 0, 128)
RED = (128, 0, 0)
GREEN = (0, 128, 0)
YELLOW = (128, 128, 0)
MAGENTA = (128, 0, 128)
CYAN = (0, 128, 128)
WHITE = (192, 192, 192)
LIGHT_BLACK = (128, 128, 128)
LIGHT_RED = (255, 0, 0)
LIGHT_GREEN = (0, 255, 0)
LIGHT_YELLOW = (255, 255, 0)
LIGHT_BLUE = (0, 0, 255)
LIGHT_MAGENTA = (255, 0, 255)

COLORS = denverapi.ctext.ColoredText.cloredTextEscapeSequenceFore
COLORS_PYGAME_LIST = [BLACK, BLUE, RED, GREEN, YELLOW, MAGENTA, CYAN, WHITE,
                      LIGHT_BLACK, LIGHT_RED, LIGHT_GREEN, LIGHT_YELLOW, LIGHT_BLUE,
                      LIGHT_MAGENTA, (0, 0, 0, 0)]
color_select_fore = 0
color_select_back = 0

ORIGINAL_WHITE = (255, 255, 255)
DARKISH_GREY = (56, 56, 56)
DARKER_GREY = (40, 40, 40)

# grid data
grid = [[None for _ in range(40)] for _ in range(80)]
grid_render_font = pygame.font.Font(f"{__file__}/../_cpic_editor/consola.ttf", 18)


def generate_color_pallet(colors: list, selected: int):
    surface = pygame.Surface((len(colors)*CELL_WIDTH, CELL_HEIGHT))
    for position, index in zip(range(0, len(colors)*CELL_WIDTH, CELL_WIDTH), range(len(colors))):
        pygame.draw.rect(surface, colors[index], pygame.Rect((position, 0), (CELL_WIDTH, CELL_HEIGHT)))
    pygame.draw.rect(surface, BLACK, pygame.Rect(selected*CELL_WIDTH, 0, CELL_WIDTH, CELL_HEIGHT), 2)
    return surface


def render_grid(surface, data: list) -> None:
    for x in range(len(data)):
        for y in range(len(data[0])):
            if data[x][y] is not None:
                color_fore, color_back, cell_text = data[x][y]
                surface_coordinates = grid_to_surface_coordinates((x, y))
                rect = pygame.Rect(surface_coordinates[0]+1, surface_coordinates[1]+1, CELL_WIDTH-1, CELL_HEIGHT-1)
                pygame.draw.rect(surface, COLORS_PYGAME_LIST[color_back], rect)
                cell_text_rendered = grid_render_font.render(cell_text, True, COLORS_PYGAME_LIST[color_fore])
                cell_text_rendered_rect: pygame.Rect = cell_text_rendered.get_rect()
                cell_text_rendered_rect.centerx = surface_coordinates[0] + CELL_WIDTH // 2
                cell_text_rendered_rect.centery = surface_coordinates[1] + CELL_HEIGHT // 2
                surface.blit(cell_text_rendered, cell_text_rendered_rect)


def draw_grid(surface: pygame.Surface):
    height = surface.get_height()
    width = surface.get_width()
    for x in range(0, height, CELL_HEIGHT):
        pygame.draw.line(surface, ORIGINAL_WHITE, (0, x), (width, x))  # Horizontal lines
    for x in range(0, width, CELL_WIDTH):
        pygame.draw.line(surface, ORIGINAL_WHITE, (x, 0), (x, height))


def transform_surface_coordinates_to_grid_coordinates(surface_coordinates):
    return surface_coordinates[0] // CELL_WIDTH, surface_coordinates[1] // CELL_HEIGHT


def grid_to_surface_coordinates(grid_coordinates):
    return grid_coordinates[0] * CELL_WIDTH, grid_coordinates[1] * CELL_HEIGHT


def main(args):
    global color_select_fore
    global color_select_back
    global grid
    file_name = args.store
    if file_name is None:
        file_name = args.file
    if file_name is None:
        raise ValueError("Please specify either one of the options")
    font = pygame.font.Font(f"{__file__}/../_cpic_editor/consola.ttf", 13)
    fore_color_label = font.render("Fore Color", True, ORIGINAL_WHITE)
    fore_color_label_rect: pygame.Rect = fore_color_label.get_rect()
    fore_color_label_rect.midtop = (600, 2)

    back_color_label = font.render("Back Color", True, ORIGINAL_WHITE)
    back_color_label_rect: pygame.Rect = back_color_label.get_rect()
    back_color_label_rect.midtop = (600, 38)

    display = pygame.display.set_mode((WIDTH, HEIGHT))
    pygame.display.set_caption("CPic Editor")
    grid_surface = pygame.Surface((720, 480))
    clock = pygame.time.Clock()

    grid_surface_rect: pygame.Rect = grid_surface.get_rect()
    grid_surface_rect.bottom = HEIGHT

    last_selected = "fore"
    while True:
        clock.tick(FPS)
        display.fill(DARKISH_GREY)
        grid_surface.fill(DARKER_GREY)
        draw_grid(grid_surface)
        render_grid(grid_surface, grid)

        color_pallet_fore = generate_color_pallet(COLORS_PYGAME_LIST, color_select_fore)
        color_pallet_fore_rect: pygame.Rect = color_pallet_fore.get_rect()
        color_pallet_fore_rect.midtop = (600, 14)

        color_pallet_back = generate_color_pallet(COLORS_PYGAME_LIST, color_select_back)
        color_pallet_back_rect: pygame.Rect = color_pallet_back.get_rect()
        color_pallet_back_rect.midtop = (600, 50)

        display.blit(grid_surface, grid_surface_rect)
        display.blit(color_pallet_fore, color_pallet_fore_rect)
        display.blit(color_pallet_back, color_pallet_back_rect)
        display.blit(fore_color_label, fore_color_label_rect)
        display.blit(back_color_label, back_color_label_rect)
        pygame.display.update()

        mouse_position = pygame.mouse.get_pos()
        if grid_surface_rect.collidepoint(*mouse_position) and pygame.mouse.get_pressed()[0]:
            mouse_x_no_offset = mouse_position[0] - grid_surface_rect.left
            mouse_y_no_offset = mouse_position[1] - grid_surface_rect.top
            grid_coordinates = transform_surface_coordinates_to_grid_coordinates((mouse_x_no_offset,
                                                                                  mouse_y_no_offset))
            grid[grid_coordinates[0]][grid_coordinates[1]] = (color_select_fore, color_select_back, " ")

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                raise SystemExit(0)
            if event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    if last_selected == "fore":
                        color_select_fore -= 1
                    elif last_selected == "back":
                        color_select_back -= 1
                if event.key == pygame.K_RIGHT:
                    if last_selected == "fore":
                        color_select_fore += 1
                    elif last_selected == "back":
                        color_select_back += 1
            if event.type == pygame.KEYDOWN:
                if event.unicode != "":
                    mouse_position = pygame.mouse.get_pos()
                    if grid_surface_rect.collidepoint(*mouse_position):
                        mouse_x_no_offset = mouse_position[0] - grid_surface_rect.left
                        mouse_y_no_offset = mouse_position[1] - grid_surface_rect.top
                        grid_coordinates = transform_surface_coordinates_to_grid_coordinates((mouse_x_no_offset,
                                                                                              mouse_y_no_offset))
                        grid[grid_coordinates[0]][grid_coordinates[1]] = (color_select_fore, color_select_back,
                                                                          event.unicode)
            if event.type == pygame.MOUSEBUTTONUP:
                mouse_position = event.pos
                if color_pallet_fore_rect.collidepoint(*mouse_position):
                    last_selected = "fore"
                    mouse_x_no_offset = mouse_position[0]-color_pallet_fore_rect.left
                    color_select_fore = transform_surface_coordinates_to_grid_coordinates((mouse_x_no_offset, 1))[0]
                elif color_pallet_back_rect.collidepoint(*mouse_position):
                    last_selected = "back"
                    mouse_x_no_offset = mouse_position[0]-color_pallet_back_rect.left
                    color_select_back = transform_surface_coordinates_to_grid_coordinates((mouse_x_no_offset, 1))[0]

        if color_select_back >= len(COLORS_PYGAME_LIST):
            color_select_back -= len(COLORS_PYGAME_LIST)
        if color_select_fore >= len(COLORS_PYGAME_LIST):
            color_select_fore -= len(COLORS_PYGAME_LIST)
        if color_select_fore < 0:
            color_select_fore += len(COLORS_PYGAME_LIST)
        if color_select_back < 0:
            color_select_back += len(COLORS_PYGAME_LIST)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("file", help="File Name to open", nargs="?", default=None)
    parser.add_argument("-s", "--store", help="Place to save the edited file", required=False, default=None)
    arguments = parser.parse_args()

    main(arguments)



"""
Data parse

This module provides enough functionalities for visual
presentation of data such as dictionaries and lists
"""

__version__ = '2020.6.4'
__author__ = "Xcodz"


import pickle as achar  # achar is pickle in HINDI (India)
import os


# tree printers
def Tree(lis: list, level=0, indent='\t'):
    cx = ''
    for x in lis:
        if type(x) == list:
            cx += Tree(x, level + 1, indent)
        else:
            cx += (indent * level) + x + '\n'
    return cx


def Tree2(lis: list, level=0, indent='  '):
    cx = ''
    for x in lis:
        if type(x) == list:
            cx += Tree2(x, level + 1, indent)
        else:
            cx += (('|' + indent) * level) + '+--' + x + '\n'
    return cx


def Tree3(d: dict, level=0, indent='  '):
    cx = ''
    for k, v in d.items():
        if type(v) == dict:
            if v != {}:
                cx += (('|' + indent) * level) + '+--' + k + '\n'
                cx += Tree3(v, level + 1, indent)
            else:
                cx += (('|' + indent) * level) + '+--' + k + ' (EMPTY)\n'
        else:
            cx += (('|' + indent) * level) + '+--' + k + '\n'
    return cx


def StorageTree(Path) -> dict:
    import os
    stor = {}
    for r, d, f in os.walk(Path):
        nr = r[len(Path) + 1:]
        if len(d) != 0:
            for x in d:
                VSD.mkdir(os.path.join(nr, x).replace(os.sep, '/'), stor)
        if len(f) != 0:
            for x in f:
                VSD.write(os.path.join(nr, x).replace(os.sep, '/'), 'File', stor)
    return stor


# Virtual Storage Dictionary
class VSD:
    dic = {}

    def mkdir(path: str, dic: dict = dic):
        cx = ''
        for x in path.split('/'):
            cx += '[' + repr(x) + ']'
        exec('dic' + cx + ' = {}')

    def path(path: str, dic: dict = dic):
        cx = ''
        for x in path.split('/'):
            cx += '[' + repr(x) + ']'
        return eval('dic' + cx)

    def write(path: str, data, dic: dict = dic):
        cx = ''
        for x in path.split('/'):
            cx += '[' + repr(x) + ']'
        exec('dic' + cx + ' = ' + repr(data))

    def delete(path: str, dic: dict = dic):
        cx = ''
        for x in path.split('/'):
            cx += '[' + repr(x) + ']'
        exec('del dic' + cx)

    def dump(dic: dict = dic):
        return achar.dumps(dic)

    def load(data):
        return achar.loads(data)



import setuptools
from denverapi import pysetup

with open("README.md") as f:
    long_description = f.read()

setuptools.setup(
    name="denver_api",
    packages=setuptools.find_packages()+setuptools.find_namespace_packages(include=["denverapi", "denverapi.*"]),
    package_data=pysetup.find_package_data("denverapi", "denverapi"),
    version="2.2.2",
    author="xcodz",
    description="Denver API for python full-stack development",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "requests",
        "playsound",
        "pygame",
        "cryptography"
    ],
    zip_safe=False
)



"""
Colored Text for beautiful output to the screen.
Anyone can use this in python application.
Also Colorama is present here by default. and
is used. This Module is tested with windows and
is perfect for cross-platform applications
which target *nix, Windows, etc.
"""

__version__ = '2020.6.4'
__author__ = "Xcodz"

import builtins

if __name__ != '__main__':
    try:
        from . import _colorama as colorama
    except:
        import _colorama as colorama
else:
    import _colorama as colorama

import os, sys, shutil

colorama.initialise.init()


class ColoredText:
    def getTerminalSize():
        return shutil.get_terminal_size()

    def clearLine(mode=2):
        print(colorama.ansi.clear_line(mode), end="", flush=True)

    def clearScreen(mode=2):
        print(colorama.ansi.clear_screen(mode), end="", flush=True)

    def styleText(text: str = ' ', fore='none', style='none', back='none'):
        return ColoredText.escape("{fore_" + fore + "}{back_" + back + '}{style_' + style + '}' + text + '{reset_all}')

    def codeToChars(code):
        return colorama.ansi.code_to_chars(code)

    def setTitle(title):
        print(colorama.ansi.set_title(title))

    class Cursor:
        def printat(x=1, y=1, text: str = ' ', fore='none', style='none', back='none'):
            print(ColoredText.Cursor.pos(x, y) + ColoredText.styleText(text, fore=fore, back=back, style=style))

        def pos(x=1, y=1):
            print(colorama.ansi.Cursor.POS(x, y))

        def up(n=1):
            print(colorama.ansi.Cursor.UP(n))

        def down(n=1):
            print(colorama.ansi.Cursor.DOWN(n))

        def left(n=1):
            print(colorama.ansi.Cursor.BACK(n))

        def right(n=1):
            print(colorama.ansi.Cursor.FORWARD(n))

    def escape(Text: str):
        return Text.format(**ColoredText.escapeSequence)

    cloredTextEscapeSequenceBack = {
        "black": colorama.Back.BLACK,
        "blue": colorama.Back.BLUE,
        "red": colorama.Back.RED,
        "green": colorama.Back.GREEN,
        "yellow": colorama.Back.YELLOW,
        "magenta": colorama.Back.MAGENTA,
        "cyan": colorama.Back.CYAN,
        "white": colorama.Back.WHITE,
        "lightblack": colorama.Back.LIGHTBLACK_EX,
        "lightred": colorama.Back.LIGHTRED_EX,
        'lightgreen': colorama.Back.LIGHTGREEN_EX,
        "lightyellow": colorama.Back.LIGHTYELLOW_EX,
        "lightblue": colorama.Back.LIGHTBLUE_EX,
        "lightmagenta": colorama.Back.LIGHTMAGENTA_EX,
        "lightcyan": colorama.Fore.LIGHTCYAN_EX,
        "lightwhite": colorama.Fore.LIGHTWHITE_EX,
        "none": ''
    }
    cloredTextEscapeSequenceFore = {
        "black": colorama.Fore.BLACK,
        "blue": colorama.Fore.BLUE,
        "red": colorama.Fore.RED,
        "green": colorama.Fore.GREEN,
        "yellow": colorama.Fore.YELLOW,
        "magenta": colorama.Fore.MAGENTA,
        "cyan": colorama.Fore.CYAN,
        "white": colorama.Fore.WHITE,
        "lightblack": colorama.Fore.LIGHTBLACK_EX,
        "lightred": colorama.Fore.LIGHTRED_EX,
        'lightgreen': colorama.Fore.LIGHTGREEN_EX,
        "lightyellow": colorama.Fore.LIGHTYELLOW_EX,
        "lightblue": colorama.Fore.LIGHTBLUE_EX,
        "lightmagenta": colorama.Fore.LIGHTMAGENTA_EX,
        "lightcyan": colorama.Fore.LIGHTCYAN_EX,
        "lightwhite": colorama.Fore.LIGHTWHITE_EX,
        "none": ''
    }
    styleEscapeSequence = {
        "bright": colorama.Style.BRIGHT,
        "dim": colorama.Style.DIM,
        "normal": colorama.Style.NORMAL,
        "none": ''
    }
    resetEscapeSequence = {
        "fore": colorama.Fore.RESET,
        "back": colorama.Back.RESET,
        "all": colorama.Style.RESET_ALL + colorama.Back.RESET + colorama.Fore.RESET,
        "none": ''
    }
    commonEscapeSequence = {
        "none": ''
    }


def s(t, d):
    return {t + '_' + k: v for k, v in d.items()}


d = {}
d.update(
    **ColoredText.commonEscapeSequence,
    **s('reset', ColoredText.resetEscapeSequence),
    **s('style', ColoredText.styleEscapeSequence),
    **s('fore', ColoredText.cloredTextEscapeSequenceFore),
    **s('back', ColoredText.cloredTextEscapeSequenceBack))

ColoredText.escapeSequence = d

del d, s


def print(*text, fore='none', back='none', style='bright', sep=' ', end='\n', flush=False, file=sys.stdout):
    try:
        builtins.print(
            ColoredText.styleText(sep.join([x if type(x) == str else repr(x) for x in text]),
                                  fore=fore, back=back, style=style), sep=sep, flush=flush,
            file=file, end=end)
    except Exception as e:
        builtins.print(*text, sep=sep, flush=flush, file=file, end=end)


def input(prompt='', fore='none', back='none', style='bright'):
    print(prompt, fore=fore, back=back, style=style, end='', flush=True)
    return builtins.input()



"""
Raw Data

A Module for struct module for making formats
and use them directly or by conversion
"""

import struct

__version__ = "2020.6.4"
__author__ = "Xcodz"

pack_format = {
    "i8": 'b',
    "iu8": 'B',
    "i16": 'h',
    "iu16": "H",
    "i32": 'i',
    "iu32": "I",
    "i64": 'l',
    "iu64": "L",
    "i128": 'q',
    "iu128": "Q",
    "f16": 'e',
    "f32": "f",
    "f64": "d",
    "c": "c",
    "b": "?",
}


def make_struct_format(format: list):
    d = "<"
    for x in format:
        d += pack_format[x]
    return d


def pack(fmt, *objs):
    return struct.pack(make_struct_format(fmt), *objs)


def unpack(fmt, bytstr:bytes):
    return struct.unpack(make_struct_format(fmt), bytstr)


def calc_size(fmt):
    return struct.calcsize(make_struct_format(fmt))


def main():
    fmt = ["i8", "i8", "b", "c"]
    obj = (123, 12, True, b"a")

    fmts = make_struct_format(fmt)
    pf = struct.pack(fmts, *obj)

    d = unpack(("iu32",), b"\xca\x0d\0\0")

    print(fmt, obj, fmts, pf, d, sep="\n")


if __name__ == '__main__':
    main()



"""
Denver is a project targeting python developers for easy and fast development
with the modules provided within it.
"""

__author__ = 'Xcodz'
__version__ = '2020.10.23'

import sys
import subprocess
from . import (bcli,
               bdtp,
               bdtpfserv,
               bitmap,
               crypt,
               ctext,
               datau,
               datp,
               get,
               graphics3d,
               graphics2d,
               handypy,
               log,
               pysetup,
               rawd,
               sound,
               text,
               cpic)


def install_pip_package(package: str, pre=False, update=False) -> int:
    arguments = [sys.executable, "-m", "pip", "install", package]
    if pre:
        arguments.append("--pre")
    if update:
        arguments.append("--update")
    return subprocess.run([sys.executable, "-m", "pip", "install", package], stdout=sys.stdout, stdin=sys.stdin,
                          stderr=sys.stderr).returncode
